#include "cache.h"
#include "xmlrpciface.h"

#include <kdebug.h>
#include <kstandarddirs.h>

#include <qdatastream.h>
#include <qfile.h>

bool CacheEntry::isValid() const
{
	// Cache entries get invalid after on hour. One shouldn't hardcode this
	// but for now it'll do.
	return m_timeStamp.secsTo( QDateTime::currentDateTime() ) < 3600;
}

Cache *Cache::m_instance = 0;

Cache &Cache::self()
{
	if ( !m_instance )
		m_instance = new Cache;
	return *m_instance;
}

QString Cache::getCacheKey( const QString &server, const QString &method,
                            const QValueList<QVariant> &args )
{
	QString key;
	key = server + QString::fromLatin1( "__" );
	key += method + QString::fromLatin1( "__" );
	QValueList<QVariant>::ConstIterator it = args.begin();
	QValueList<QVariant>::ConstIterator end = args.end();
	for ( ; it != end; ++it )
		key += KXMLRPC::Query::marshal( *it );
	
	return key;
}

Cache::Cache()
{
	load();
}

Cache::~Cache()
{
	save();
}

void Cache::load()
{
	QFile file( cacheFileName() );
	if ( !file.open( IO_ReadOnly ) ) {
		kdDebug() << "Failed to open cache file " << cacheFileName() << endl;
		return;
	}
	
	QDataStream stream( &file );
	while ( !stream.atEnd() ) {
		QString key;
		stream >> key;

		CacheEntry *entry = new CacheEntry;
		stream >> *entry;

		QDict<CacheEntry>::insert( key, entry );
	}
}

void Cache::save()
{
	QFile file( cacheFileName() );
	if ( !file.open( IO_WriteOnly ) ) {
		kdDebug() << "Failed to open cache file " << cacheFileName() << endl;
		return;
	}

	QDataStream stream( &file );

	QDictIterator<CacheEntry> it( *this );
	for ( ; it.current() != 0; ++it )
		stream << it.currentKey() << *it.current();
}

void Cache::touch( const QString &key )
{
	CacheEntry *entry = find( key );
	if ( !entry )
		return;
	entry->m_timeStamp = QDateTime::currentDateTime();
}

void Cache::insert( const QString &key, const KXMLRPC::Query::Result &result )
{
	CacheEntry *entry = new CacheEntry;
	entry->m_timeStamp = QDateTime::currentDateTime();
	entry->m_result = result;
	QDict<CacheEntry>::insert( key, entry );
}

QString Cache::cacheFileName() const
{
	return locateLocal( "appdata", "cache/dcoprss.cache" );
}

