#ifndef CACHE_H
#define CACHE_H

#include <qcstring.h>
#include <qdatetime.h>
#include <qdict.h>

#include <xmlrpciface.h>

class CacheEntry
{
	friend class Cache;
	friend QDataStream &operator>>( QDataStream &s, CacheEntry &e );
	public:
		const QDateTime &timeStamp() const { return m_timeStamp; }
		const KXMLRPC::Query::Result result() const { return m_result; }
		bool isValid() const;
	
	private:
		QDateTime m_timeStamp;
		KXMLRPC::Query::Result m_result;
};

class Cache : public QDict<CacheEntry>
{
	public:
		static Cache &self();

		static QString getCacheKey( const QString &server,
		                            const QString &method,
		                            const QValueList<QVariant> &args );

		void load();
		void save();

		void touch( const QString &key );

		void insert( const QString &key, const KXMLRPC::Query::Result &result );

	private:
		Cache();
		Cache( const Cache &rhs ); // disabled
		Cache &operator=( const Cache &rhs ); // disabled
		~Cache();

		QString cacheFileName() const;

		static Cache *m_instance;
};

inline QDataStream &operator<<( QDataStream &s, const CacheEntry &e )
{
	return s << e.timeStamp() << e.result();
}

inline QDataStream &operator>>( QDataStream &s, CacheEntry &e )
{
	return s >> e.m_timeStamp >> e.m_result;
}

#endif // CACHE_H
// vim:ts=4:sw=4:noet
